/* NOTE:
 * This file was modified for DOSEMU by the DOSEMU-team.
 * The original is 'Copyright 1997 Willows Software, Inc.' and generously
 * was put under the GNU Library General Public License.
 * ( for more information see http://www.willows.com/ )
 *
 * We make use of section 3 of the GNU Library General Public License
 * ('...opt to apply the terms of the ordinary GNU General Public License...'),
 * because the resulting product is an integrated part of DOSEMU and
 * can not be considered to be a 'library' in the terms of Library License.
 * The (below) original copyright notice from Willows therefore was edited
 * conforming to section 3 of the GNU Library General Public License.
 *
 * Nov. 1 1997, The DOSEMU team.
 */


/*************************************************************************

	@(#)mod_rm.h	1.4
    	Copyright 1997 Willows Software, Inc. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; see the file COPYING.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.


For more information about the Willows Twin Libraries.

	http://www.willows.com	

To send email to the maintainer of the Willows Twin Libraries.

	mailto:twin@willows.com 

**************************************************************************/

#ifndef mod_rm__h
#define mod_rm__h

#define	MOD_AX_BXSI	0x00
#define	MOD_AX_BXDI	0x01
#define	MOD_AX_BPSI	0x02
#define	MOD_AX_BPDI	0x03
#define	MOD_AX_SI	0x04
#define	MOD_AX_DI	0x05
#define	MOD_AX_imm16	0x06
#define	MOD_AX_BX	0x07
#define	MOD_CX_BXSI	0x08
#define	MOD_CX_BXDI	0x09
#define	MOD_CX_BPSI	0x0a
#define	MOD_CX_BPDI	0x0b
#define	MOD_CX_SI	0x0c
#define	MOD_CX_DI	0x0d
#define	MOD_CX_imm16	0x0e
#define	MOD_CX_BX	0x0f
#define	MOD_DX_BXSI	0x10
#define	MOD_DX_BXDI	0x11
#define	MOD_DX_BPSI	0x12
#define	MOD_DX_BPDI	0x13
#define	MOD_DX_SI	0x14
#define	MOD_DX_DI	0x15
#define	MOD_DX_imm16	0x16
#define	MOD_DX_BX	0x17
#define	MOD_BX_BXSI	0x18
#define	MOD_BX_BXDI	0x19
#define	MOD_BX_BPSI	0x1a
#define	MOD_BX_BPDI	0x1b
#define	MOD_BX_SI	0x1c
#define	MOD_BX_DI	0x1d
#define	MOD_BX_imm16	0x1e
#define	MOD_BX_BX	0x1f
#define	MOD_SP_BXSI	0x20
#define	MOD_SP_BXDI	0x21
#define	MOD_SP_BPSI	0x22
#define	MOD_SP_BPDI	0x23
#define	MOD_SP_SI	0x24
#define	MOD_SP_DI	0x25
#define	MOD_SP_imm16	0x26
#define	MOD_SP_BX	0x27
#define	MOD_BP_BXSI	0x28
#define	MOD_BP_BXDI	0x29
#define	MOD_BP_BPSI	0x2a
#define	MOD_BP_BPDI	0x2b
#define	MOD_BP_SI	0x2c
#define	MOD_BP_DI	0x2d
#define	MOD_BP_imm16	0x2e
#define	MOD_BP_BX	0x2f
#define	MOD_SI_BXSI	0x30
#define	MOD_SI_BXDI	0x31
#define	MOD_SI_BPSI	0x32
#define	MOD_SI_BPDI	0x33
#define	MOD_SI_SI	0x34
#define	MOD_SI_DI	0x35
#define	MOD_SI_imm16	0x36
#define	MOD_SI_BX	0x37
#define	MOD_DI_BXSI	0x38
#define	MOD_DI_BXDI	0x39
#define	MOD_DI_BPSI	0x3a
#define	MOD_DI_BPDI	0x3b
#define	MOD_DI_SI	0x3c
#define	MOD_DI_DI	0x3d
#define	MOD_DI_imm16	0x3e
#define	MOD_DI_BX	0x3f
#define	MOD_AX_BXSIimm8	0x40
#define	MOD_AX_BXDIimm8	0x41
#define	MOD_AX_BPSIimm8	0x42
#define	MOD_AX_BPDIimm8	0x43
#define	MOD_AX_SIimm8	0x44
#define	MOD_AX_DIimm8	0x45
#define	MOD_AX_BPimm8	0x46
#define	MOD_AX_BXimm8	0x47
#define	MOD_CX_BXSIimm8	0x48
#define	MOD_CX_BXDIimm8	0x49
#define	MOD_CX_BPSIimm8	0x4a
#define	MOD_CX_BPDIimm8	0x4b
#define	MOD_CX_SIimm8	0x4c
#define	MOD_CX_DIimm8	0x4d
#define	MOD_CX_BPimm8	0x4e
#define	MOD_CX_BXimm8	0x4f
#define	MOD_DX_BXSIimm8	0x50
#define	MOD_DX_BXDIimm8	0x51
#define	MOD_DX_BPSIimm8	0x52
#define	MOD_DX_BPDIimm8	0x53
#define	MOD_DX_SIimm8	0x54
#define	MOD_DX_DIimm8	0x55
#define	MOD_DX_BPimm8	0x56
#define	MOD_DX_BXimm8	0x57
#define	MOD_BX_BXSIimm8	0x58
#define	MOD_BX_BXDIimm8	0x59
#define	MOD_BX_BPSIimm8	0x5a
#define	MOD_BX_BPDIimm8	0x5b
#define	MOD_BX_SIimm8	0x5c
#define	MOD_BX_DIimm8	0x5d
#define	MOD_BX_BPimm8	0x5e
#define	MOD_BX_BXimm8	0x5f
#define	MOD_SP_BXSIimm8	0x60
#define	MOD_SP_BXDIimm8	0x61
#define	MOD_SP_BPSIimm8	0x62
#define	MOD_SP_BPDIimm8	0x63
#define	MOD_SP_SIimm8	0x64
#define	MOD_SP_DIimm8	0x65
#define	MOD_SP_BPimm8	0x66
#define	MOD_SP_BXimm8	0x67
#define	MOD_BP_BXSIimm8	0x68
#define	MOD_BP_BXDIimm8	0x69
#define	MOD_BP_BPSIimm8	0x6a
#define	MOD_BP_BPDIimm8	0x6b
#define	MOD_BP_SIimm8	0x6c
#define	MOD_BP_DIimm8	0x6d
#define	MOD_BP_BPimm8	0x6e
#define	MOD_BP_BXimm8	0x6f
#define	MOD_SI_BXSIimm8	0x70
#define	MOD_SI_BXDIimm8	0x71
#define	MOD_SI_BPSIimm8	0x72
#define	MOD_SI_BPDIimm8	0x73
#define	MOD_SI_SIimm8	0x74
#define	MOD_SI_DIimm8	0x75
#define	MOD_SI_BPimm8	0x76
#define	MOD_SI_BXimm8	0x77
#define	MOD_DI_BXSIimm8	0x78
#define	MOD_DI_BXDIimm8	0x79
#define	MOD_DI_BPSIimm8	0x7a
#define	MOD_DI_BPDIimm8	0x7b
#define	MOD_DI_SIimm8	0x7c
#define	MOD_DI_DIimm8	0x7d
#define	MOD_DI_BPimm8	0x7e
#define	MOD_DI_BXimm8	0x7f
#define	MOD_AX_BXSIimm16	0x80
#define	MOD_AX_BXDIimm16	0x81
#define	MOD_AX_BPSIimm16	0x82
#define	MOD_AX_BPDIimm16	0x83
#define	MOD_AX_SIimm16	0x84
#define	MOD_AX_DIimm16	0x85
#define	MOD_AX_BPimm16	0x86
#define	MOD_AX_BXimm16	0x87
#define	MOD_CX_BXSIimm16	0x88
#define	MOD_CX_BXDIimm16	0x89
#define	MOD_CX_BPSIimm16	0x8a
#define	MOD_CX_BPDIimm16	0x8b
#define	MOD_CX_SIimm16	0x8c
#define	MOD_CX_DIimm16	0x8d
#define	MOD_CX_BPimm16	0x8e
#define	MOD_CX_BXimm16	0x8f
#define	MOD_DX_BXSIimm16	0x90
#define	MOD_DX_BXDIimm16	0x91
#define	MOD_DX_BPSIimm16	0x92
#define	MOD_DX_BPDIimm16	0x93
#define	MOD_DX_SIimm16	0x94
#define	MOD_DX_DIimm16	0x95
#define	MOD_DX_BPimm16	0x96
#define	MOD_DX_BXimm16	0x97
#define	MOD_BX_BXSIimm16	0x98
#define	MOD_BX_BXDIimm16	0x99
#define	MOD_BX_BPSIimm16	0x9a
#define	MOD_BX_BPDIimm16	0x9b
#define	MOD_BX_SIimm16	0x9c
#define	MOD_BX_DIimm16	0x9d
#define	MOD_BX_BPimm16	0x9e
#define	MOD_BX_BXimm16	0x9f
#define	MOD_SP_BXSIimm16	0xa0
#define	MOD_SP_BXDIimm16	0xa1
#define	MOD_SP_BPSIimm16	0xa2
#define	MOD_SP_BPDIimm16	0xa3
#define	MOD_SP_SIimm16	0xa4
#define	MOD_SP_DIimm16	0xa5
#define	MOD_SP_BPimm16	0xa6
#define	MOD_SP_BXimm16	0xa7
#define	MOD_BP_BXSIimm16	0xa8
#define	MOD_BP_BXDIimm16	0xa9
#define	MOD_BP_BPSIimm16	0xaa
#define	MOD_BP_BPDIimm16	0xab
#define	MOD_BP_SIimm16	0xac
#define	MOD_BP_DIimm16	0xad
#define	MOD_BP_BPimm16	0xae
#define	MOD_BP_BXimm16	0xaf
#define	MOD_SI_BXSIimm16	0xb0
#define	MOD_SI_BXDIimm16	0xb1
#define	MOD_SI_BPSIimm16	0xb2
#define	MOD_SI_BPDIimm16	0xb3
#define	MOD_SI_SIimm16	0xb4
#define	MOD_SI_DIimm16	0xb5
#define	MOD_SI_BPimm16	0xb6
#define	MOD_SI_BXimm16	0xb7
#define	MOD_DI_BXSIimm16	0xb8
#define	MOD_DI_BXDIimm16	0xb9
#define	MOD_DI_BPSIimm16	0xba
#define	MOD_DI_BPDIimm16	0xbb
#define	MOD_DI_SIimm16	0xbc
#define	MOD_DI_DIimm16	0xbd
#define	MOD_DI_BPimm16	0xbe
#define	MOD_DI_BXimm16	0xbf
#define	MOD_AX_EW_AX	0xc0
#define	MOD_AX_EW_CX	0xc1
#define	MOD_AX_EW_DX	0xc2
#define	MOD_AX_EW_BX	0xc3
#define	MOD_AX_EW_SP	0xc4
#define	MOD_AX_EW_BP	0xc5
#define	MOD_AX_EW_SI	0xc6
#define	MOD_AX_EW_DI	0xc7
#define	MOD_CX_EW_AX	0xc8
#define	MOD_CX_EW_CX	0xc9
#define	MOD_CX_EW_DX	0xca
#define	MOD_CX_EW_BX	0xcb
#define	MOD_CX_EW_SP	0xcc
#define	MOD_CX_EW_BP	0xcd
#define	MOD_CX_EW_SI	0xce
#define	MOD_CX_EW_DI	0xcf
#define	MOD_DX_EW_AX	0xd0
#define	MOD_DX_EW_CX	0xd1
#define	MOD_DX_EW_DX	0xd2
#define	MOD_DX_EW_BX	0xd3
#define	MOD_DX_EW_SP	0xd4
#define	MOD_DX_EW_BP	0xd5
#define	MOD_DX_EW_SI	0xd6
#define	MOD_DX_EW_DI	0xd7
#define	MOD_BX_EW_AX	0xd8
#define	MOD_BX_EW_CX	0xd9
#define	MOD_BX_EW_DX	0xda
#define	MOD_BX_EW_BX	0xdb
#define	MOD_BX_EW_SP	0xdc
#define	MOD_BX_EW_BP	0xdd
#define	MOD_BX_EW_SI	0xde
#define	MOD_BX_EW_DI	0xdf
#define	MOD_SP_EW_AX	0xe0
#define	MOD_SP_EW_CX	0xe1
#define	MOD_SP_EW_DX	0xe2
#define	MOD_SP_EW_BX	0xe3
#define	MOD_SP_EW_SP	0xe4
#define	MOD_SP_EW_BP	0xe5
#define	MOD_SP_EW_SI	0xe6
#define	MOD_SP_EW_DI	0xe7
#define	MOD_BP_EW_AX	0xe8
#define	MOD_BP_EW_CX	0xe9
#define	MOD_BP_EW_DX	0xea
#define	MOD_BP_EW_BX	0xeb
#define	MOD_BP_EW_SP	0xec
#define	MOD_BP_EW_BP	0xed
#define	MOD_BP_EW_SI	0xee
#define	MOD_BP_EW_DI	0xef
#define	MOD_SI_EW_AX	0xf0
#define	MOD_SI_EW_CX	0xf1
#define	MOD_SI_EW_DX	0xf2
#define	MOD_SI_EW_BX	0xf3
#define	MOD_SI_EW_SP	0xf4
#define	MOD_SI_EW_BP	0xf5
#define	MOD_SI_EW_SI	0xf6
#define	MOD_SI_EW_DI	0xf7
#define	MOD_DI_EW_AX	0xf8
#define	MOD_DI_EW_CX	0xf9
#define	MOD_DI_EW_DX	0xfa
#define	MOD_DI_EW_BX	0xfb
#define	MOD_DI_EW_SP	0xfc
#define	MOD_DI_EW_BP	0xfd
#define	MOD_DI_EW_SI	0xfe
#define	MOD_DI_EW_DI	0xff
#define	MOD_AL_BXSI	0x00
#define	MOD_AL_BXDI	0x01
#define	MOD_AL_BPSI	0x02
#define	MOD_AL_BPDI	0x03
#define	MOD_AL_SI	0x04
#define	MOD_AL_DI	0x05
#define	MOD_AL_imm16	0x06
#define	MOD_AL_BX	0x07
#define	MOD_CL_BXSI	0x08
#define	MOD_CL_BXDI	0x09
#define	MOD_CL_BPSI	0x0a
#define	MOD_CL_BPDI	0x0b
#define	MOD_CL_SI	0x0c
#define	MOD_CL_DI	0x0d
#define	MOD_CL_imm16	0x0e
#define	MOD_CL_BX	0x0f
#define	MOD_DL_BXSI	0x10
#define	MOD_DL_BXDI	0x11
#define	MOD_DL_BPSI	0x12
#define	MOD_DL_BPDI	0x13
#define	MOD_DL_SI	0x14
#define	MOD_DL_DI	0x15
#define	MOD_DL_imm16	0x16
#define	MOD_DL_BX	0x17
#define	MOD_BL_BXSI	0x18
#define	MOD_BL_BXDI	0x19
#define	MOD_BL_BPSI	0x1a
#define	MOD_BL_BPDI	0x1b
#define	MOD_BL_SI	0x1c
#define	MOD_BL_DI	0x1d
#define	MOD_BL_imm16	0x1e
#define	MOD_BL_BX	0x1f
#define	MOD_AH_BXSI	0x20
#define	MOD_AH_BXDI	0x21
#define	MOD_AH_BPSI	0x22
#define	MOD_AH_BPDI	0x23
#define	MOD_AH_SI	0x24
#define	MOD_AH_DI	0x25
#define	MOD_AH_imm16	0x26
#define	MOD_AH_BX	0x27
#define	MOD_CH_BXSI	0x28
#define	MOD_CH_BXDI	0x29
#define	MOD_CH_BPSI	0x2a
#define	MOD_CH_BPDI	0x2b
#define	MOD_CH_SI	0x2c
#define	MOD_CH_DI	0x2d
#define	MOD_CH_imm16	0x2e
#define	MOD_CH_BX	0x2f
#define	MOD_DH_BXSI	0x30
#define	MOD_DH_BXDI	0x31
#define	MOD_DH_BPSI	0x32
#define	MOD_DH_BPDI	0x33
#define	MOD_DH_SI	0x34
#define	MOD_DH_DI	0x35
#define	MOD_DH_imm16	0x36
#define	MOD_DH_BX	0x37
#define	MOD_BH_BXSI	0x38
#define	MOD_BH_BXDI	0x39
#define	MOD_BH_BPSI	0x3a
#define	MOD_BH_BPDI	0x3b
#define	MOD_BH_SI	0x3c
#define	MOD_BH_DI	0x3d
#define	MOD_BH_imm16	0x3e
#define	MOD_BH_BX	0x3f
#define	MOD_AL_BXSIimm8	0x40
#define	MOD_AL_BXDIimm8	0x41
#define	MOD_AL_BPSIimm8	0x42
#define	MOD_AL_BPDIimm8	0x43
#define	MOD_AL_SIimm8	0x44
#define	MOD_AL_DIimm8	0x45
#define	MOD_AL_BPimm8	0x46
#define	MOD_AL_BXimm8	0x47
#define	MOD_CL_BXSIimm8	0x48
#define	MOD_CL_BXDIimm8	0x49
#define	MOD_CL_BPSIimm8	0x4a
#define	MOD_CL_BPDIimm8	0x4b
#define	MOD_CL_SIimm8	0x4c
#define	MOD_CL_DIimm8	0x4d
#define	MOD_CL_BPimm8	0x4e
#define	MOD_CL_BXimm8	0x4f
#define	MOD_DL_BXSIimm8	0x50
#define	MOD_DL_BXDIimm8	0x51
#define	MOD_DL_BPSIimm8	0x52
#define	MOD_DL_BPDIimm8	0x53
#define	MOD_DL_SIimm8	0x54
#define	MOD_DL_DIimm8	0x55
#define	MOD_DL_BPimm8	0x56
#define	MOD_DL_BXimm8	0x57
#define	MOD_BL_BXSIimm8	0x58
#define	MOD_BL_BXDIimm8	0x59
#define	MOD_BL_BPSIimm8	0x5a
#define	MOD_BL_BPDIimm8	0x5b
#define	MOD_BL_SIimm8	0x5c
#define	MOD_BL_DIimm8	0x5d
#define	MOD_BL_BPimm8	0x5e
#define	MOD_BL_BXimm8	0x5f
#define	MOD_AH_BXSIimm8	0x60
#define	MOD_AH_BXDIimm8	0x61
#define	MOD_AH_BPSIimm8	0x62
#define	MOD_AH_BPDIimm8	0x63
#define	MOD_AH_SIimm8	0x64
#define	MOD_AH_DIimm8	0x65
#define	MOD_AH_BPimm8	0x66
#define	MOD_AH_BXimm8	0x67
#define	MOD_CH_BXSIimm8	0x68
#define	MOD_CH_BXDIimm8	0x69
#define	MOD_CH_BPSIimm8	0x6a
#define	MOD_CH_BPDIimm8	0x6b
#define	MOD_CH_SIimm8	0x6c
#define	MOD_CH_DIimm8	0x6d
#define	MOD_CH_BPimm8	0x6e
#define	MOD_CH_BXimm8	0x6f
#define	MOD_DH_BXSIimm8	0x70
#define	MOD_DH_BXDIimm8	0x71
#define	MOD_DH_BPSIimm8	0x72
#define	MOD_DH_BPDIimm8	0x73
#define	MOD_DH_SIimm8	0x74
#define	MOD_DH_DIimm8	0x75
#define	MOD_DH_BPimm8	0x76
#define	MOD_DH_BXimm8	0x77
#define	MOD_BH_BXSIimm8	0x78
#define	MOD_BH_BXDIimm8	0x79
#define	MOD_BH_BPSIimm8	0x7a
#define	MOD_BH_BPDIimm8	0x7b
#define	MOD_BH_SIimm8	0x7c
#define	MOD_BH_DIimm8	0x7d
#define	MOD_BH_BPimm8	0x7e
#define	MOD_BH_BXimm8	0x7f
#define	MOD_AL_BXSIimm16	0x80
#define	MOD_AL_BXDIimm16	0x81
#define	MOD_AL_BPSIimm16	0x82
#define	MOD_AL_BPDIimm16	0x83
#define	MOD_AL_SIimm16	0x84
#define	MOD_AL_DIimm16	0x85
#define	MOD_AL_BPimm16	0x86
#define	MOD_AL_BXimm16	0x87
#define	MOD_CL_BXSIimm16	0x88
#define	MOD_CL_BXDIimm16	0x89
#define	MOD_CL_BPSIimm16	0x8a
#define	MOD_CL_BPDIimm16	0x8b
#define	MOD_CL_SIimm16	0x8c
#define	MOD_CL_DIimm16	0x8d
#define	MOD_CL_BPimm16	0x8e
#define	MOD_CL_BXimm16	0x8f
#define	MOD_DL_BXSIimm16	0x90
#define	MOD_DL_BXDIimm16	0x91
#define	MOD_DL_BPSIimm16	0x92
#define	MOD_DL_BPDIimm16	0x93
#define	MOD_DL_SIimm16	0x94
#define	MOD_DL_DIimm16	0x95
#define	MOD_DL_BPimm16	0x96
#define	MOD_DL_BXimm16	0x97
#define	MOD_BL_BXSIimm16	0x98
#define	MOD_BL_BXDIimm16	0x99
#define	MOD_BL_BPSIimm16	0x9a
#define	MOD_BL_BPDIimm16	0x9b
#define	MOD_BL_SIimm16	0x9c
#define	MOD_BL_DIimm16	0x9d
#define	MOD_BL_BPimm16	0x9e
#define	MOD_BL_BXimm16	0x9f
#define	MOD_AH_BXSIimm16	0xa0
#define	MOD_AH_BXDIimm16	0xa1
#define	MOD_AH_BPSIimm16	0xa2
#define	MOD_AH_BPDIimm16	0xa3
#define	MOD_AH_SIimm16	0xa4
#define	MOD_AH_DIimm16	0xa5
#define	MOD_AH_BPimm16	0xa6
#define	MOD_AH_BXimm16	0xa7
#define	MOD_CH_BXSIimm16	0xa8
#define	MOD_CH_BXDIimm16	0xa9
#define	MOD_CH_BPSIimm16	0xaa
#define	MOD_CH_BPDIimm16	0xab
#define	MOD_CH_SIimm16	0xac
#define	MOD_CH_DIimm16	0xad
#define	MOD_CH_BPimm16	0xae
#define	MOD_CH_BXimm16	0xaf
#define	MOD_DH_BXSIimm16	0xb0
#define	MOD_DH_BXDIimm16	0xb1
#define	MOD_DH_BPSIimm16	0xb2
#define	MOD_DH_BPDIimm16	0xb3
#define	MOD_DH_SIimm16	0xb4
#define	MOD_DH_DIimm16	0xb5
#define	MOD_DH_BPimm16	0xb6
#define	MOD_DH_BXimm16	0xb7
#define	MOD_BH_BXSIimm16	0xb8
#define	MOD_BH_BXDIimm16	0xb9
#define	MOD_BH_BPSIimm16	0xba
#define	MOD_BH_BPDIimm16	0xbb
#define	MOD_BH_SIimm16	0xbc
#define	MOD_BH_DIimm16	0xbd
#define	MOD_BH_BPimm16	0xbe
#define	MOD_BH_BXimm16	0xbf
#define	MOD_AL_EB_AL	0xc0
#define	MOD_AL_EB_CL	0xc1
#define	MOD_AL_EB_DL	0xc2
#define	MOD_AL_EB_BL	0xc3
#define	MOD_AL_EB_AH	0xc4
#define	MOD_AL_EB_CH	0xc5
#define	MOD_AL_EB_DH	0xc6
#define	MOD_AL_EB_BH	0xc7
#define	MOD_CL_EB_AL	0xc8
#define	MOD_CL_EB_CL	0xc9
#define	MOD_CL_EB_DL	0xca
#define	MOD_CL_EB_BL	0xcb
#define	MOD_CL_EB_AH	0xcc
#define	MOD_CL_EB_CH	0xcd
#define	MOD_CL_EB_DH	0xce
#define	MOD_CL_EB_BH	0xcf
#define	MOD_DL_EB_AL	0xd0
#define	MOD_DL_EB_CL	0xd1
#define	MOD_DL_EB_DL	0xd2
#define	MOD_DL_EB_BL	0xd3
#define	MOD_DL_EB_AH	0xd4
#define	MOD_DL_EB_CH	0xd5
#define	MOD_DL_EB_DH	0xd6
#define	MOD_DL_EB_BH	0xd7
#define	MOD_BL_EB_AL	0xd8
#define	MOD_BL_EB_CL	0xd9
#define	MOD_BL_EB_DL	0xda
#define	MOD_BL_EB_BL	0xdb
#define	MOD_BL_EB_AH	0xdc
#define	MOD_BL_EB_CH	0xdd
#define	MOD_BL_EB_DH	0xde
#define	MOD_BL_EB_BH	0xdf
#define	MOD_AH_EB_AL	0xe0
#define	MOD_AH_EB_CL	0xe1
#define	MOD_AH_EB_DL	0xe2
#define	MOD_AH_EB_BL	0xe3
#define	MOD_AH_EB_AH	0xe4
#define	MOD_AH_EB_CH	0xe5
#define	MOD_AH_EB_DH	0xe6
#define	MOD_AH_EB_BH	0xe7
#define	MOD_CH_EB_AL	0xe8
#define	MOD_CH_EB_CL	0xe9
#define	MOD_CH_EB_DL	0xea
#define	MOD_CH_EB_BL	0xeb
#define	MOD_CH_EB_AH	0xec
#define	MOD_CH_EB_CH	0xed
#define	MOD_CH_EB_DH	0xee
#define	MOD_CH_EB_BH	0xef
#define	MOD_DH_EB_AL	0xf0
#define	MOD_DH_EB_CL	0xf1
#define	MOD_DH_EB_DL	0xf2
#define	MOD_DH_EB_BL	0xf3
#define	MOD_DH_EB_AH	0xf4
#define	MOD_DH_EB_CH	0xf5
#define	MOD_DH_EB_DH	0xf6
#define	MOD_DH_EB_BH	0xf7
#define	MOD_BH_EB_AL	0xf8
#define	MOD_BH_EB_CL	0xf9
#define	MOD_BH_EB_DL	0xfa
#define	MOD_BH_EB_BL	0xfb
#define	MOD_BH_EB_AH	0xfc
#define	MOD_BH_EB_CH	0xfd
#define	MOD_BH_EB_DH	0xfe
#define	MOD_BH_EB_BH	0xff
#endif /* mod_rm__h */
