/* 
 * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
 *
 * for details see file COPYING in the DOSEMU distribution
 */

/*
 * NetBSD sound code Header
 */

/*
 * NetBSD sound code Header
 */

